#!/usr/bin/perl
use strict;
use warnings;
use Test::More;

eval "use Test::ConsistentVersion";
plan skip_all => "Test::ConsistentVersion required "
    . "for checking module versions" if $@;
Test::ConsistentVersion::check_consistent_versions(
    # We don't use the version in the README
    no_readme => 1,
    # We don't maintain a ChangeLog file, we use NEWS instead
    no_changelog => 1,
    # Test::Pod::Content lookѕ for VERSION blocks in POD, which (currently)
    # don't exist in Glib's autogenerated POD files
    no_pod => 1,
);
